import { DictCode } from '@/constants';

import type { Component, ComponentPublicInstance } from 'vue';
import type { StepProps, UploadProps } from 'ant-design-vue';
import type { Rule, RuleObject } from 'ant-design-vue/es/form';
import type { ProtocolConfigMethod } from '@/constants';

export interface ApiResponse<T> {
  data: T;
  code: number;
  msg: string;
}

export interface PageData<T> {
  records: T[];
  total: number;
  pageIndex: number;
  pageSize: number;
  pageNum: number;
}

export interface PageParams {
  pageIndex: number;
  pageSize: number;
  pageSorts?: {
    column: string;
    asc: boolean;
  }[];
}

export interface OptionItem<T> {
  value: T;
  label: string;
}

export type CheckedType = boolean | string | number;

export type FormRules<T> = {
  [K in keyof T]?: Rule[];
} & {
  [key: string]: Rule[];
};

export type RuleValidator<T> = (rule: RuleObject, value: T, callback: (error?: string) => void) => Promise<void>;

export interface DictTypeDataParams {
  id?: number;
  dictCode?: DictCode;
}

export interface DictTypeData {
  dictId: number;
  dictCode: string;
  dictName: string;
  dictEngName: string;
  dictOrderNum: number | null;
  dictTypeDataList: DictValue[];
}

export interface DictValue {
  dictValueId: number;
  dictValue: string;
  dictEngValue: string;
  dictValueOrderNum: number;
  isDefault: boolean | null;
}

export interface UseGuideStepItem extends StepProps {
  title: string;
  component: Component;
  contentOffset?: number;
  formLayout?: 'horizontal' | 'vertical' | 'inline';
  isLastStep?: boolean;
  exportButtonShow?: boolean;
  nextStepButtonText?: string;
  nextStepButtonDisabled?: boolean;
}

export interface UseGuideStepItemProps<T> {
  form: T;
  steps: UseGuideStepItem[];
  stepIndex: number;
}

export type UseGuideStepItemExpose = {
  exportData?: () => void | Promise<void>;
  finish?: () => void | Promise<void>;
};

export type UseGuideStepItemInstance = ComponentPublicInstance<unknown, UseGuideStepItemExpose>;

export interface SetupProtocolForm {
  protocolType: string;
  protocolInfo: ProtocolBaseInfo;
  protocolFile: UploadProps['fileList'];
  configMethod: ProtocolConfigMethod;
  deviceType: string;
}

export interface CustomParamsForm {
  gatewayParamName: string;
  gatewayParamCode: string;
  unit: string;
  module: string;
  readWriteType: string;
  parsingType: string;
  writeFunctionCode: string;
  readFunctionCode: string;
  addressLength: number;
  registerAddress: number;
  coefficient: number;
  isHighFrequencyParameter: string;
  readCalculationFormula: string;
  writeCalculationFormula: string;
  decimalPlaces: number;
  contiguousAddressingRange: number;
}

export interface ProtocolBaseInfo {
  id?: number;
  protocolName: string;
  protocolType: string;
  deviceType: string;
  deviceTypeId: number | null;
  dataBit: 5 | 6 | 7 | 8;
  parityBit: 'N' | 'O' | 'E';
  stopBit: '1' | '1.5' | '2';
  baudRate: number;
  dataSendInterval: number;
  highFreqSendInterval: number;
  readTimeout: number;
  nextDataReadDelay: number;
  nextRoundDataReadDelay: number;
  readContinuousAddr: string;
  readContinuousAddrCode: number;
  readContinuousAddrLength: number;
  byteOrder: string;
  byteOrderCode: string;
  addrOrder: string;
  addrOrderCode: string;
}

export type ProtocolParamData = PageData<ProtocolParamInfo>;

export interface ProtocolParamInfo {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  baseInfoId: number;
  platformParamCode: string;
  platformParamName: string;
  gatewayParamCode: string;
  gatewayParamName: string;
  paramCode: string;
  paramName: string;
  unit: string;
  module: string;
  readWriteType: string;
  readWriteTypeCode: string;
  readFuncCode: string;
  writeFuncCode: string;
  registerAddr: string;
  addrNumber: string;
  registerType: string;
  registerTypeCode: string;
  parsingType: string;
  parsingTypeCode: string;
  addrLength: number;
  wordLength: number;
  wordLengthCode: string;
  quantity: number;
  coefficient: number;
  isHighFreqParam: string;
  isHighFreqParamCode: string;
  readCalcFormula: string;
  writeCalcFormula: string;
  decimalPlace: number;
}

export interface ProtocolParamSearchParams extends PageParams {
  baseInfoId: number;
  paramCode?: string;
  paramName?: string;
}

export type ProtocolStandardParamData = PageData<ProtocolStandardParam>;

export interface ProtocolStandardParam {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  paramCode: string;
  paramName: string;
  unit: string;
  module: string;
  deviceTypeId: number;
  deviceType: string;
  enabled: string;
}

export interface ProtocolStandardParamSearchParams extends PageParams {
  paramCode?: string;
  paramName?: string;
}

export interface RegisterGatewayForm {
  snCode: string;
  password: string;
  modelId: number;
  linkType: string;
  id: number;
}

export interface VerificationEquipment {
  snCode: string;
  password: string;
}

export interface GatewayInformation {
  createTime: string;
  createUserId: string;
  deleted: number;
  id: number;
  isRegister: number;
  modelId: number;
  password: string;
  snCode: string;
  state: number;
  updateTime: string;
  updateUserId: number;
}

export interface GatewayModelInfo {
  createTime: string;
  createUserId: string;
  deleted: number;
  docUrl: string;
  iconUrl: string;
  id: number;
  interfaceId: string;
  interfaceNum: string;
  modelName: string;
  surfMode: string;
  surfModeEn: string;
  updateTime: string;
  updateUserId: string;
}

export interface InterfaceNum {
  COM: number;
  DI: number;
  DO: number;
  LAN: number;
  WAN: number;
}

export interface EquipmentInformation {
  id: number;
  state: number;
  modelId: number;
  docUrl: string;
  iconUrl: string;
  interfaceNum: string;
  modelName: string;
  surfMode: string | string[];
  surfModeEn: string | string[];
}

export interface GatewayInterface {
  comment: string;
  createTime: string;
  createUserId: string;
  deleted: number;
  dictCode: string;
  dictEngValue: string;
  dictValue: string;
  enabled: number;
  id: number;
  isDefault: string;
  orderNum: number;
  tag: string;
  updateTime: string;
  updateUserId: string;
}

export interface AddInterface {
  linkName: string;
  gatewayId: number;
  linkTypeId: number;
}

export interface InterfaceList {
  value: string;
  label: string;
}

export interface InterfaceData {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  linkName: string;
  gatewayId: number;
  linkTypeId: number;
  protocolType: string;
  bindState: number;
  dataBit: number;
  parityBit: string;
  stopBit: string;
  baudRate: number;
  readTimeout: number;
  nextDataReadDelay: number;
  nextRoundDataReadDelay: number;
  valueType: string;
}

export interface AgreementUpdate {
  id: number;
  linkName: string;
  gatewayId: number;
  linkTypeId: number;
  bindState: number;
  dataBit: string;
  parityBit: string;
  stopBit: string;
  baudRate: number;
  readTimeout: number;
  nextDataReadDelay: number;
  nextRoundDataReadDelay: number;
}

export interface AgreementData {
  protocolType: string;
  station: string;
  name: string;
  protocolName: string;
  dev: string;
  dataSendInterval: number;
  highFreqSendInterval: number;
  protocolId: number;
  isStandard: string;
}

export interface AgreementForm {
  baudRate: number;
  dataBit: string;
  parityBit: string;
  stopBit: string;
  readTimeout: number;
  nextDataReadDelay: number;
  nextRoundDataReadDelay: number;
}

export interface EquipmentGroup {
  pageIndex: number;
  pageSize: number;
  content: string;
}

export interface DeviceGroupItem {
  id: number;
  createTime: Date | null;
  updateTime: Date | null;
  createUserId: number | null;
  updateUserId: number | null;
  groupName: string;
  comment: string | null;
  parentId: number;
  userId: number;
  deleted: boolean | null;
  deviceGroupChilds: DeviceGroupChild[];
}

export interface DeviceGroupChild {
  id: number;
  groupName: string;
  comment: string | null;
  parentId: number;
  userId: number;
}

export interface ListInfo {
  dataType: number;
}

export interface ProtocolList {
  pageIndex: number;
  pageSize: number;
}

export interface EquipmentInformationForm {
  groupId: number;
  deviceData: number[];
  deviceType: string;
  deviceName: string;
  brand: number;
  model: number;
  modelType: number;
  controlType: number;
  compressionLevel: number;
  voltageLevel: number;
  powerRating: string;
  powerUnload: string;
  maximumFlow: string;
  pressure: string;
  pressureMax: string;
  productionDate: string;
  productionNum: string;
  deviceNum: string;
  mainTechData: string;
  mountedPosition: string;
  department: string;
  respPerson: string;
  phone: string;
  serviceLife: number;
  status: string;
  remarks: string;
  devId: number;
}

export interface EquipmentTypeItem {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  dataType: number;
  dataName: string;
}

export type ProtocolItemData = PageData<ProtocolItem>;
export interface ProtocolItem {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  protocolName: string;
  protocolType: string;
  deviceTypeId: number | null;
  deviceType: string;
  dataBit: number;
  parityBit: string;
  stopBit: string;
  baudRate: number;
  dataSendInterval: number;
  highFreqSendInterval: number;
  readTimeout: number | null;
  nextDataReadDelay: number;
  nextRoundDataReadDelay: number;
  addrOrder: string;
  addrOrderCode: string;
  byteOrder: string;
  byteOrderCode: string;
  attachment: string;
  readContinuousAddr: string;
  readContinuousAddrCode: number;
  readContinuousAddrLength: number;
}

export interface ProtocolsItem {
  /*协议ID */
  protocolId: number;

  /*从站号 */
  station: string;

  /*是否平台标准协议 */
  isStandard: string;

  /*普通数据上报频率 */
  dataSendInterval: number;

  /*高频发送间隔 */
  highFreqSendInterval: number;
}
export interface ProtocolReset {
  linkId: number;
  protocols: ProtocolsItem[];
}

export interface VerificationAgreement {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  linkId: number;
  protocolId: number;
  station: number;
  isStandard: number;
  dataSendInterval: number;
  highFreqSendInterval: number;
  protocolName: string;
  protocolType: string;
}

export interface ProtocolListItem {
  pageIndex: number;
  pageSize: number;
  id?: number;
  linkId?: number;
  readWriteTypeCode?: string;
}

export type ProtocolListPageItemData = PageData<ProtocolListPageItem>;
export interface ProtocolListPageItem {
  id: number;
  createTime: string | null;
  updateTime: string | null;
  createUserId: number | null;
  updateUserId: number | null;
  baseInfoId: number | null;
  platformParamCode: string | null;
  platformParamName: string | null;
  gatewayParamCode: string;
  gatewayParamName: string;
  paramCode: string;
  paramName: string;
  unit: string | null;
  module: number;
  readWriteType: string | null;
  readWriteTypeCode: string | null;
  readFuncCode: string | null;
  writeFuncCode: string | null;
  registerAddr: string | null;
  addrNumber: number | null;
  registerType: string | null;
  registerTypeCode: string | null;
  parsingType: string | null;
  parsingTypeCode: string | null;
  addrLength: number | null;
  wordLength: number | null;
  wordLengthCode: string | null;
  quantity: number | null;
  coefficient: number;
  isHighFreqParam: boolean | null;
  isHighFreqParamCode: string | null;
  readCalcFormula: string | null;
  writeCalcFormula: string | null;
  decimalPlace: number | null;
}

export type SerialNumberItemData = PageData<SerialNumberItem>;
export interface SerialNumberItem {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number | null;
  updateUserId: number | null;
  snCode: string;
  password: string;
  modelId: number;
  isRegister: number;
  state: number;
  deleted: number;
  modelName: string;
}

export type ListEquipmentParametersItemData = PageData<ListEquipmentParametersItem>;

export interface ListEquipmentParametersItem {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  baseInfoId: number;
  platformParamCode: string | null;
  platformParamName: string | null;
  gatewayParamCode: string | null;
  gatewayParamName: string | null;
  paramCode: string;
  paramName: string;
  unit: string | null;
  module: string;
  readWriteType: string;
  readWriteTypeCode: string;
  readFuncCode: string;
  writeFuncCode: string | null;
  registerAddr: string;
  addrNumber: string | null;
  registerType: string | null;
  registerTypeCode: string | null;
  parsingType: string;
  parsingTypeCode: string;
  addrLength: number;
  wordLength: number | null;
  wordLengthCode: string | null;
  quantity: number | null;
  coefficient: number;
  isHighFreqParam: string;
  isHighFreqParamCode: string;
  readCalcFormula: string | null;
  writeCalcFormula: string | null;
  decimalPlace: number | null;
}

export interface EquipmentUpdateForm {
  deviceId: number;
  gatewayId: number;
  gatewayProtocolId: number;
}

export interface PostProtocolPage {
  pageIndex: number;
  pageSize: number;
  deviceIds: number[];
  gatewayIds: number[];
}

export type PostProtocolPageItemData = PageData<PostProtocolPageItem>;
export interface PostProtocolPageItem {
  id: number;
  createTime: string;
  updateTime: string;
  createUserId: number;
  updateUserId: number;
  baseInfoId: number;
  platformParamCode?: string;
  platformParamName?: string;
  gatewayParamCode?: string;
  gatewayParamName?: string;
  paramCode: string;
  paramName: string;
  unit?: string;
  module: string;
  readWriteType: string;
  readWriteTypeCode: string;
  readFuncCode: string;
  writeFuncCode?: string;
  registerAddr: string;
  addrNumber?: string;
  registerType: string;
  registerTypeCode?: string;
  parsingType: string;
  parsingTypeCode: string;
  addrLength: number;
  wordLength?: string;
  wordLengthCode?: string;
  quantity?: string;
  coefficient: number;
  isHighFreqParam: string;
  isHighFreqParamCode: string;
  readCalcFormula?: string;
  writeCalcFormula?: string;
  decimalPlace?: number;
}

export interface ProtocolGatewayListListItem {
  pageIndex: number;
  pageSize: number;
  snCode?: number;
  gatewayId?: number;
}

export interface ParameterVerification {
  id: number;
  deviceParamGroupName: string;
  valueVos: ParameterVerificationList[];
}

export interface ParameterVerificationList {
  baseInfoId: number;
  platformParamCode: string;
  platformParamName: string;
  gatewayParamCode: string;
  gatewayParamName: string;
  paramCode: string;
  paramName: string;
  unit: string;
  module: string;
  readWriteType: string;
  readWriteTypeCode: string;
  readFuncCode: string;
  writeFuncCode: string;
  registerAddr: string;
  addrNumber: string;
  registerType: string;
  registerTypeCode: string;
  parsingType: string;
  parsingTypeCode: string;
  addrLength: number;
  wordLength: string;
  wordLengthCode: string;
  quantity: number;
  coefficient: number;
  isHighFreqParam: string;
  isHighFreqParamCode: string;
  readCalcFormula: string;
  writeCalcFormula: string;
  decimalPlace: number;
  valueId: number;
  value: number | string;
}
